#! /usr/bin/env python3
"""Run the model and return the diffraction intensities.

More information regarding this model is found in [1-3].

[1] https://arxiv.org/pdf/1903.12128.pdf
[2] https://arxiv.org/pdf/1707.08467.pdf
[3] https://arxiv.org/pdf/1910.14435.pdf

"""
import os,sys
import pathlib
import numpy as np
import itertools

from forward_problem.modules.keys import keys as default_keys
from forward_problem.modules.process_results import process_results
from forward_problem.modules.load_data import load_data

jcm_root = os.getenv('JCMROOT')
if jcm_root is None:
    raise EnvironmentError('No environment variable JCMROOT found. '+
                           'Maybe JCMsuite is not installed.')
sys.path.insert(0,os.path.join(jcm_root, 'ThirdPartySupport', 'Python'))                        
import jcmwave


class ForwardProblem:
    def __init__(self, NThreads=1, Multiplicity=2):
        self._data = load_data()

        # Start daemon at local machine
        # For more options, see https://jcmwave.com/docs/PythonInterface/53063d75ad54373ab2b55cd756fce540.html
        jcmwave.daemon.shutdown()
        jcmwave.daemon.add_workstation(
            Hostname = "localhost",
            NThreads = NThreads,
            Multiplicity = Multiplicity) 

    def exp_data(self):
        intensities = dict(
            S = dict(
                phi_0 = self._data[1]["intensities"],
                phi_90 = self._data[3]["intensities"]
            ),
            P =  dict(
                phi_0 = self._data[0]["intensities"],
                phi_90 = self._data[2]["intensities"]
            )
        )

        uncertainties = dict(
            S = dict(
                phi_0 = 0.002 + 0.02 * self._data[1]["intensities"],
                phi_90 = 0.002 + 0.02 * self._data[3]["intensities"]
            ),
            P =  dict(
                phi_0 = 0.002 + 0.02 * self._data[0]["intensities"],
                phi_90 = 0.002 + 0.02 * self._data[2]["intensities"]
            )
        )        
        
        thetas = self._data[0]["thetas"]
        
        return intensities, uncertainties, thetas

    def model_eval(self, keys, working_dir=None):
        """Run the numerical model and return the simulation results in a dictionary.
         
        :param keys: Argument keys for the simulation.
        :param working_dir: Directory, where simulation results are stored.
                     (default: temporary directory is used)

        """

        calc_keys = default_keys.copy()
        calc_keys.update(keys)
        
        import_results = load_data()

        thetas = self._data[0]["thetas"]
        phis = set(r["phi"] for r in self._data)

        t_p_combinations = list(itertools.product(thetas, phis))

        job_ids = []

        # perform the calculations
        for i, (theta, phi) in enumerate(t_p_combinations):
            these_calc_keys = calc_keys.copy()
            these_calc_keys["theta"] = theta
            these_calc_keys["phi"] = phi
            if working_dir is not None:
                wd = os.path.join(working_dir,'phi{}_theta{}'.format(phi,theta))
            else:
                wd = None
                
            job_id = jcmwave.solve(
                "forward_problem/jcm/project.jcmpt",
                keys=these_calc_keys,
                temporary=(working_dir is None),
                working_dir=wd
            )
            job_ids.append(job_id)

        # wait for the calculations
        results, logs = jcmwave.daemon.wait(job_ids=job_ids, verbose=False)

        # create dictionary for the results
        sim_res = dict()
        sim_res["I_S"] = list()
        sim_res["I_P"] = list()

        for idx, result in enumerate(results):
            try: fres = process_results(result, calc_keys)
            except:
                print('Cannot process results:')
                print(result)
                print(logs[idx])
                raise 

            sim_res["I_S"].append(fres["S"]["I"])
            sim_res["I_P"].append(fres["P"]["I"])

        # create a dictionary to store the results in
        out_quant = ["Iexp", "Isim"]

        intensities = dict(
            S = dict(
                phi_0 = sim_res["I_S"][0::2],
                phi_90 = sim_res["I_S"][1::2]
            ),
            P =  dict(
                phi_0 = sim_res["I_P"][0::2],
                phi_90 = sim_res["I_P"][1::2]
            )
        )
        return intensities
