#! /usr/bin/env python3
"""Function to process the results that were obtained from the FEM
simulations.

"""
import sys
import numpy as np


def process_results(result, keys):
    """Process the calculated results. """
    
    calc_res = result[1]

    epsi = calc_res["header"]["RelPermittivityIncoming"]
    epso = calc_res["header"]["RelPermittivity"]

    n_in = np.sqrt(epsi)
    n_out = np.sqrt(epso)

    if calc_res["header"]["RelPermeability"] != 1:
        sys.exit("Permeability should be 1")

    kin = calc_res["header"]["IncomingPlaneWaveKVector"][0]
    vacuum_wl = 2 * np.pi * n_in / np.linalg.norm(kin)

    k_in = kin.real.flatten()

    r_theta_in = np.arccos(-k_in[1] / np.linalg.norm(k_in))
    theta_in = np.rad2deg(r_theta_in)
    phi_in = np.arctan2(k_in[0], k_in[2]) * 180 / np.pi

    if calc_res["header"]["NormalDirection"] != "Y":
        sys.exit("Works only for NormalDirection Y")

    orders_n1 = calc_res["N1"]
    orders_n2 = calc_res["N2"]
    zero_order_index = np.argwhere((orders_n1 == 0 & orders_n2 == 0))
    if len(zero_order_index) != 1:
        sys.exit("Issues with zero orders")

    efields = calc_res["ElectricFieldStrength"]
    if len(efields) != 2:
        sys.exit("Issues with electric fields")

    # outgoing zero diffraction order
    k_out = calc_res["K"][zero_order_index].flatten()
    r_theta_out = np.arccos(k_out[1] / np.linalg.norm(k_out))  # in radians
    phi_out = np.arctan2(k_out[0], k_out[2]) * 180 / np.pi

    # keep the information per polarization
    polarization_data = dict()

    for polarization_index in [0, 1]:  # S and P polarization

        if polarization_index == 0:
            label = "S"
        else:
            label = "P"

        polarization_data[label] = dict()

        intensity_in = (
            np.sum(
                calc_res["header"]["IncomingPlaneWaveAmplitude"][
                    polarization_index
                ]
                ** 2
            )
            * n_in
            * np.cos(r_theta_in)
        )
        efield = efields[polarization_index][zero_order_index]

        intensity = np.sum(efield * efield.conj()).real

        normalized_intensity = intensity * n_out * np.cos(r_theta_out) / intensity_in
        normalized_intensity = normalized_intensity.real

        polarization_data[label]["I"] = normalized_intensity
        
    return polarization_data

